#!/usr/bin/perl
#
# MAP2XPM v1.1 - by Miguel Farah <miguel@lj.cl>
#
# This Perl script receives an (uncompressed!) Freeciv savegame from the
# standard input, and writes an XPM image of the terrain map. A zoom
# level (an integer >=1) can be specified in the command line (if not, a
# default zoom level of 1 is used).
#
# The output filename is of the form <year>.xpm, where the year is
# padded to the left with zeroes if necessary.
#
# If you want to generate a GIF file, you'll need to feed the generated
# file to xpmtoppm, and its output to ppmtogif.
#
# Version history:
#
# 1.0  - 17/02/2001: initial release.
# 1.01 - 20/02/2001: changed the color for the swamp terrain (the
#                    previous cyan #009080 was too close to the
#                    grassland green and it was a bit hard to
#                    distinguish both terrains in certain conditions).
# 1.1  - 02/06/2002: handle V2 maps properly (i.e.: recognize rivers).
#                    Also dumped the old "map2gif" name, as it wasn't
#                    right.
#

# Before anything else, get the zoom level.
#
$zoom=(!defined($ARGV[0]))?1:$ARGV[0];
die("ERROR: zoom level must be at least 1.\n") if ($zoom<1);
die("ERROR: zoom level must be an integer number.\n") if ($zoom!=int($zoom));


# Read the stuff we need from the savegame file. Die if critical errors
# are detected.
#
$EOS=0;
while (!$EOS) {
   $l=<STDIN>;
   $EOS=1 if (!defined($l));

   $year=$1      if ($l =~ /^year=(.*)/);          # the year (no, really)
   $width=$1     if ($l =~ /^width=(.*)/);         # map's width
   $height=$1    if ($l =~ /^height=(.*)/);        # map's heigth
   $map{$1}=$2   if ($l =~ /^t([0-9]+)="(.*)"/);   # terrain
   $river{$1}=$2 if ($l =~ /^n([0-9]+)="(.*)"/);   # river overlay
}
die("ERROR: No year defined in the savegame.\n")       if (!defined($year));
die("ERROR: No map width defined in the savegame.\n")  if (!defined($width));
die("ERROR: No map height defined in the savegame.\n") if (!defined($height));


# Open the output file. Pad zeroes to the left on its name if necessary
# (this will make easier to sort multiple files).
#
if ($year>=0) {
   $file=substr("0000".$year,-4,4);
} else {
   $year*=-1;
   $file="-".substr("0000".$year,-4,4);
}


# NOTE: to generate a .GIF file instead of an .XPM, change the line
# below to:
# open (PIPE, "| xpmtoppm | ppmtogif > $file.gif")
#
open(PIPE, "> $file.xpm")
     or die("ERROR: couldn't open the pipe to create $file.xpm.\n");


# Print the XPM header.
#
$w=$width*$zoom;
$h=$height*$zoom;

print PIPE "/* XPM */\n";
print PIPE "static char * map_xpm[] = {\n";
print PIPE '"', " $w $h 25 1 ", '"', "\n";
print PIPE '"       c #0000C0",' , "\n";
print PIPE '"g      c #009000",' , "\n";    # lowercase = terrain
print PIPE '"p      c #D0A000",' , "\n";
print PIPE '"d      c #A0A060",' , "\n";
print PIPE '"t      c #999999",' , "\n";
print PIPE '"a      c #FFFFFF",' , "\n";
print PIPE '"f      c #705000",' , "\n";
print PIPE '"j      c #50E050",' , "\n";
print PIPE '"s      c #108060",' , "\n";
print PIPE '"h      c #406040",' , "\n";
print PIPE '"m      c #545454",' , "\n";
print PIPE '"X      c #FF00FF",' , "\n";
print PIPE '"G      c #008000",' , "\n";    # uppercase = terrain+river
print PIPE '"P      c #C09000",' , "\n";
print PIPE '"D      c #909050",' , "\n";
print PIPE '"T      c #898989",' , "\n";
print PIPE '"A      c #EFEFEF",' , "\n";
print PIPE '"F      c #604000",' , "\n";
print PIPE '"J      c #40D040",' , "\n";
print PIPE '"S      c #107050",' , "\n";
print PIPE '"H      c #305030",' , "\n";
print PIPE '"M      c #444444",' , "\n";
print PIPE '"r      c #000000",' , "\n";    # river (v1)
print PIPE '"R      c #A00000",' , "\n";    # river (v1) + river (v2) (!)
print PIPE '"u      c #FF0000"';         # The comma and \n AREN'T missing.


# Print the map.
#
for ($i=0;$i<$height;$i++) {
   $a=substr("000".$i,-3,3);

   $line="";
   for ($j=1;$j<=length($map{$a});$j++) {

      if ( (((substr $river{$a}, $j-1, 1)>>1)&1) == 1) {
         # This square has a river on it.
         $c=uc(substr($map{$a},$j-1,1));
         $c="X" if ($c eq " "); # a river on an ocean square (not good)
      } else {
         # This square does not have a river on it.
         $c=substr($map{$a},$j-1,1);
      }
      $c="u" if (index(" gpdtafjshmXGPDTAFJSHMrRu", $c)<0);

      for ($k=1;$k<=$zoom;$k++) {
         $line.=$c;
      }
   }

   for ($j=1;$j<=$zoom;$j++) {
      print PIPE ",\n", '"', $line, '"';
   }

}


# Close the XPM and the pipe.
#
print PIPE "};\n";
close PIPE or die("ERROR: couldn't close the pipe.\n");


# And that's it.

